#------------------------------------------------------------------------------
#
#  Copyright (c) 2005-2013, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Thanks for using Enthought open source!
#
#  Author: David C. Morrill
#  Date:   10/07/2004
#
#------------------------------------------------------------------------------



from ._version import full_version as __version__

__requires__ = [
    'traits',
    'pyface>=6.0.0',
]
__extras_require__ = {
    'wx': ['wxpython>=2.8.10', 'numpy'],
    'pyqt': ['pyqt>=4.10', 'pygments'],
    'pyqt5': ['pyqt>=5', 'pygments'],
    'pyside': ['pyside>=1.2', 'pygments'],
}



#
#  Copyright (c) 2015, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#
#  Author: Senganal T.
#  Date:   Nov 2015
#

""" Test the storing/restoration of split group state.
"""

import nose

from traits.api import Int
from traitsui.api import Action, Group, Handler, HSplit, Item, View
from traitsui.tests._tools import skip_if_not_qt4


class TmpClass(Handler):
    aa = Int(10)
    bb = Int(100)

    def init(self, ui_info):
        super(TmpClass, self).init(ui_info)
        self.save_prefs(ui_info)

    def reset_prefs(self, ui_info):
        """ Reset the split to be equally wide.
        """
        control = getattr(ui_info, 'h_split').control
        width = control.width()
        control.moveSplitter(width/2, 1)

    def restore_prefs(self, ui_info):
        """ Apply the last saved ui preferences.
        """
        ui_info.ui.set_prefs(self._prefs)

    def save_prefs(self, ui_info):
        """ Save the current ui preferences.
        """
        self._prefs = ui_info.ui.get_prefs()

    def collapse_right(self, ui_info):
        """ Collapse the split to the right.
        """
        control = getattr(ui_info, 'h_split').control
        width = control.width()
        control.moveSplitter(width, 1)

    def collapse_left(self, ui_info):
        """ Collapse the split to the left.
        """
        control = getattr(ui_info, 'h_split').control
        control.moveSplitter(0, 1)

    view = View(
        HSplit(
            Group(
                Item('aa', resizable=True, width=50), show_border=True,
            ),
            Group(
                Item('bb', width=100), show_border=True,
            ),
            id='h_split',
        ),
        resizable=True,
        # add actions to test manually.
        buttons=[Action(name='collapse left', action='collapse_left'),
                 Action(name='collapse right', action='collapse_right'),
                 Action(name='reset_layout', action='reset_prefs'),
                 Action(name='restore layout', action='restore_prefs'),
                 Action(name='save layout', action='save_prefs')],
        height=300,
        id='test_view_for_splitter_pref_restore',
    )


@skip_if_not_qt4
def test_splitter_prefs_are_restored():
    # the keys for the splitter prefs (i.e. prefs['h_split']['structure'])
    splitter_keys = ('h_split', 'structure')

    def _get_nattr(obj, attr_names=splitter_keys):
        """ Utility function to get a value from a nested dict.
        """
        if obj is None or len(attr_names) == 0:
            return obj
        return _get_nattr(obj.get(attr_names[0], None),
                          attr_names=attr_names[1:])

    ui = TmpClass().edit_traits()
    handler = ui.handler

    # set the layout to a known state
    handler.reset_prefs(ui.info)

    # save the current layout and check (sanity test)
    handler.save_prefs(ui.info)
    nose.tools.assert_equal(_get_nattr(handler._prefs),
                            _get_nattr(ui.get_prefs()))

    # collapse splitter to right and check prefs has been updated
    handler.collapse_right(ui.info)
    nose.tools.assert_not_equal(_get_nattr(handler._prefs),
                                _get_nattr(ui.get_prefs()))

    # restore the original layout.
    handler.restore_prefs(ui.info)
    nose.tools.assert_equal(_get_nattr(handler._prefs),
                            _get_nattr(ui.get_prefs()))

    # collapse to left and check
    handler.collapse_left(ui.info)
    nose.tools.assert_not_equal(_get_nattr(handler._prefs),
                                _get_nattr(ui.get_prefs()))

    # save the collapsed layout
    handler.save_prefs(ui.info)
    collapsed_splitter_state = _get_nattr(handler._prefs)
    nose.tools.assert_equal(_get_nattr(handler._prefs),
                            _get_nattr(ui.get_prefs()))

    # dispose the ui.
    ui.dispose()

    # create a new ui and check that the splitter remembers the last state
    # (collapsed)
    ui2 = TmpClass().edit_traits()
    nose.tools.assert_equal(collapsed_splitter_state,
                            _get_nattr(ui2.get_prefs()))


if __name__ == '__main__':
    # Execute from command line for manual testing
    # start a session to modify the default layout.
    TmpClass().configure_traits()

    # check visually if the layout changes from the prev. session is
    # restored.
    TmpClass().configure_traits()



#------------------------------------------------------------------------------
#
#  Copyright (c) 2017, Enthought, Inc.
#  All rights reserved.
#
#  This software is provided without warranty under the terms of the BSD
#  license included in enthought/LICENSE.txt and may be redistributed only
#  under the conditions described in the aforementioned license.  The license
#  is also available online at http://www.enthought.com/licenses/BSD.txt
#------------------------------------------------------------------------------

import contextlib
import unittest

from traits.api import Event, HasTraits, Instance, Int, List, Unicode
from traits.testing.api import UnittestTools

from traitsui.api import Item, TabularEditor, View
from traitsui.tabular_adapter import TabularAdapter
from traitsui.tests._tools import skip_if_null


class Person(HasTraits):
    name = Unicode
    age = Int

    def __repr__(self):
        return "Person(name={self.name!r}, age={self.age!r})".format(self=self)

class ReportAdapter(TabularAdapter):
    columns = [
        ('Name', 'name'),
        ('Age', 'age'),
    ]


class Report(HasTraits):
    people = List(Person)

    selected = Instance(Person)

    selected_row = Int(-1)

    # Event for triggering a UI repaint.
    refresh = Event

    # Event for triggering a UI table update.
    update = Event

    traits_view = View(
        Item(
            name='people',
            editor=TabularEditor(
                adapter=ReportAdapter(),
                selected='selected',
                selected_row='selected_row',
                refresh='refresh',
                update='update',
            ),
        ),
    )


class TestTabularEditor(UnittestTools, unittest.TestCase):

    @skip_if_null
    def test_selected_reacts_to_model_changes(self):
        with self.report_and_editor() as (report, editor):
            people = report.people

            self.assertIsNone(report.selected)
            self.assertEqual(report.selected_row, -1)

            report.selected = people[1]
            self.assertEqual(report.selected, people[1])
            self.assertEqual(report.selected_row, 1)

            report.selected = None
            self.assertIsNone(report.selected)
            self.assertEqual(report.selected_row, -1)

            report.selected_row = 0
            self.assertEqual(report.selected, people[0])
            self.assertEqual(report.selected_row, 0)

            report.selected_row = -1
            self.assertIsNone(report.selected)
            self.assertEqual(report.selected_row, -1)

    @skip_if_null
    def test_event_synchronization(self):
        with self.report_and_editor() as (report, editor):
            with self.assertTraitChanges(editor, 'refresh', count=1):
                report.refresh = True
            # Should happen every time.
            with self.assertTraitChanges(editor, 'refresh', count=1):
                report.refresh = True

            with self.assertTraitChanges(editor, 'update', count=1):
                report.update = True
            with self.assertTraitChanges(editor, 'update', count=1):
                report.update = True

    @contextlib.contextmanager
    def report_and_editor(self):
        """
        Context manager to temporarily create and clean up a Report model object
        and the corresponding TabularEditor.
        """
        report = Report(
            people=[
                Person(name='Theresa', age=60),
                Person(name='Arlene', age=46),
            ],
        )
        ui = report.edit_traits()
        try:
            editor, = ui.get_editors('people')
            yield report, editor
        finally:
            ui.dispose()
